@@ -4,18 +4,25 @@ Arkindex API Client
 import os
 import apistar
 import logging
+import requests
 import warnings
 import yaml
 from arkindex.auth import TokenSessionAuthentication
 from arkindex.encoders import XMLEncoder
+from arkindex.exceptions import SchemaError
 from arkindex.pagination import ResponsePaginator
 from arkindex.transports import ArkindexHTTPTransport
 from io import IOBase, BufferedIOBase
 from time import sleep
-from urllib.parse import urlsplit, urlunsplit
+from urllib.parse import urlsplit, urlunsplit, urljoin
 
 BASE_DIR = os.path.dirname(os.path.abspath(__file__))
 
+DEFAULT_BASE_URL = 'https://arkindex.teklia.com/'
+
+# Endpoint accessed by the client on instanciation to retrieve the OpenAPI schema
+SCHEMA_ENDPOINT = '/api/v1/openapi/?format=openapi-json'
+
 logger = logging.getLogger(__name__)
 
 
@@ -27,11 +34,12 @@ def options_from_env():
     if 'ARKINDEX_API_TOKEN' in os.environ:
         options['token'] = os.environ.get('ARKINDEX_API_TOKEN')
 
-    if 'ARKINDEX_API_INTERNAL_URL' in os.environ:
-        options['base_url'] = os.environ.get('ARKINDEX_API_INTERNAL_URL')
-    elif 'ARKINDEX_API_URL' in os.environ:
+    if 'ARKINDEX_API_URL' in os.environ:
         options['base_url'] = os.environ.get('ARKINDEX_API_URL')
 
+    if 'ARKINDEX_API_SCHEMA_URL' in os.environ:
+        options['schema_url'] = os.environ.get('ARKINDEX_API_SCHEMA_URL')
+
     if 'ARKINDEX_API_CSRF_COOKIE' in os.environ:
         options['csrf_cookie'] = os.environ.get('ARKINDEX_API_CSRF_COOKIE')
 
@@ -51,12 +59,13 @@ class ArkindexClient(apistar.Client):
     An Arkindex API client.
     """
 
-    def __init__(self, token=None, base_url=None, csrf_cookie=None, sleep=0, **kwargs):
+    def __init__(self, token=None, base_url=DEFAULT_BASE_URL, schema_url=None, csrf_cookie=None, sleep=0, **kwargs):
         r"""
         :param token: An API token to use. If omitted, access is restricted to public endpoints.
         :type token: str or None
-        :param base_url: A custom base URL for the client. If omitted, defaults to the Arkindex main server.
-        :type base_url: str or None
+        :param str base_url: A custom base URL for the client. If omitted, defaults to the Arkindex main server.
+        :param schema_url: URL or local path to an OpenAPI schema to use instead of the Arkindex instance's own schema.
+        :type schema_url: str or None
         :param csrf_cookie: Use a custom CSRF cookie name. By default, the client will try to use any cookie
            defined in ``x-csrf-cookie`` on the Server Object of the OpenAPI specification, and fall back to
            ``arkindex.csrf``.
@@ -65,11 +74,28 @@ class ArkindexClient(apistar.Client):
            as a simple means of throttling.
         :param \**kwargs: Keyword arguments to send to ``apistar.Client``.
         """
-        with open(os.path.join(BASE_DIR, 'schema.yml')) as f:
-            schema = yaml.safe_load(f.read())
+        if not schema_url:
+            schema_url = urljoin(base_url, SCHEMA_ENDPOINT)
+
+        try:
+            split = urlsplit(schema_url)
+            if split.scheme == 'file' or not (split.scheme or split.netloc):
+                # This is a local path
+                with open(schema_url) as f:
+                    schema = yaml.safe_load(f)
+            else:
+                resp = requests.get(schema_url)
+                resp.raise_for_status()
+                schema = yaml.safe_load(resp.content)
+        except Exception as e:
+            raise SchemaError(f'Could not retrieve a proper OpenAPI schema from {schema_url}') from e
 
         super().__init__(schema, **kwargs)
 
+        # APIStar will treat a schema as valid even when there are no endpoints, making the client completely useless.
+        if not len(self.document.walk_links()):
+            raise SchemaError(f'The OpenAPI schema from {base_url} has no defined endpoints')
+
         # Post-processing of the parsed schema
         for link_info in self.document.walk_links():
             # Look for deprecated links
@@ -98,7 +124,10 @@ class ArkindexClient(apistar.Client):
         self.configure(token=token, base_url=base_url, csrf_cookie=csrf_cookie, sleep=sleep)
 
     def __repr__(self):
-        return '<{} on {}>'.format(self.__class__.__name__, self.document.url)
+        return '<{} on {!r}>'.format(
+            self.__class__.__name__,
+            self.document.url if hasattr(self, 'document') else '',
+        )
 
     def init_transport(self, *args, **kwargs):
         return ArkindexHTTPTransport(*args, **kwargs)
